use crate::components::header::Header;
use crate::models::HoldPosition;
use crate::models::Wall;
use leptos::ev::Event;
use leptos::html::Input;
use leptos::prelude::*;
use serde::Deserialize;
use serde::Serialize;
use wasm_bindgen::JsCast;
use wasm_bindgen::prelude::*;
use web_sys::FileList;

#[component]
pub fn EditWall() -> impl leptos::IntoView {
    let load = async move {
        // TODO: What to do about this unwrap?
        load_initial_data().await.unwrap()
    };

    leptos::view! {
        <div class="min-w-screen min-h-screen bg-slate-900">
            <div class="container mx-auto">
                <Header />
                <Await future=load let:data>
                    <Ready data=data.to_owned() />
                </Await>
            </div>
        </div>
    }
}

#[component]
fn Ready(data: InitialData) -> impl leptos::IntoView {
    leptos::logging::log!("ready");
    let mut hold_positions = vec![];
    for row in 0..(data.wall.rows) {
        for col in 0..(data.wall.cols) {
            hold_positions.push(HoldPosition { row, col });
        }
    }

    let mut holds = vec![];
    for &hold_position in &hold_positions {
        holds.push(view! { <Hold hold_position /> });
    }

    let grid_classes = format!("grid grid-rows-{} grid-cols-{} gap-4", data.wall.rows, data.wall.cols);

    view! { <div class=move || { grid_classes.clone() }>{holds}</div> }
}

#[component]
fn Hold(hold_position: HoldPosition) -> impl leptos::IntoView {
    let file_input_ref = NodeRef::<Input>::new();

    let open_camera = move |_| {
        if let Some(input) = file_input_ref.get() {
            input.click(); // Trigger the file input click programmatically
        }
    };

    // Callback to handle file selection
    let on_file_input = move |event: Event| {
        let files: FileList = event.target().unwrap().unchecked_ref::<web_sys::HtmlInputElement>().files().unwrap();
        leptos::logging::log!("{:?}", &files);
        let file = files.item(0).unwrap();
        leptos::logging::log!("{:?}", &file);

        let file_reader = web_sys::FileReader::new().unwrap();
        file_reader.read_as_array_buffer(&file).unwrap();

        leptos::logging::log!("foo");

        let onload = Closure::wrap(Box::new(move |event: Event| {
            leptos::logging::log!("onload");
            let file_reader: web_sys::FileReader = event.target().unwrap().dyn_into().unwrap();
            let file = file_reader.result().unwrap();
            let file = web_sys::js_sys::Uint8Array::new(&file);

            let mut file_buffer = vec![0; file.length() as usize];
            file.copy_to(&mut file_buffer);

            leptos::logging::log!("bytes: {:?}", &file_buffer.len());
        }) as Box<dyn FnMut(_)>);

        file_reader.set_onload(Some(onload.as_ref().unchecked_ref()));
        onload.forget();
    };

    view! {
        <button on:click=open_camera>
            <div class="bg-indigo-100 aspect-square rounded"></div>
        </button>

        <input
            node_ref=file_input_ref
            type="file"
            accept="image/*"
            capture="user"
            style="display: none;"
            on:input=on_file_input
        />
    }
}

#[derive(Serialize, Deserialize, Clone)]
pub struct InitialData {
    wall: Wall,
}

#[server]
async fn load_initial_data() -> Result<InitialData, ServerFnError> {
    use crate::server::state::State;

    let state = expect_context::<State>();

    let wall = state.persistent.with(|s| s.wall.clone()).await;
    Ok(InitialData { wall })
}
